/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2002 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : alleg/sound.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou avril 1999
 *  Modifi par: Eric Botcazou 24/09/2001
 *
 *  Gestion de l'mulation sonore du TO7-70.
 */


#ifndef SCAN_DEPEND
   #include <stdio.h>
   #include <string.h>
   #include <allegro.h>
#endif

#include "alleg/main.h"
#include "to7.h"


static AUDIOSTREAM *stream;
static int sound_freq;
static int sound_buffer_size;
static unsigned char *sound_buffer;
static int last_index;
static unsigned char last_data;



/* StartSound:
 *  Lance le streaming audio.
 */
void StartSound(void)
{
    voice_start(stream->voice);
}



/* StopSound:
 *  Arrte le streaming audio.
 */
void StopSound(void)
{
    voice_stop(stream->voice);
}



/* GetVolume:
 *  Lit le volume du streaming audio.
 */
int GetVolume(void)
{
    return voice_get_volume(stream->voice);
}



/* SetVolume:
 *  Fixe le volume du streaming audio.
 */
void SetVolume(int val)
{
    voice_set_volume(stream->voice, val);
}



/* PutSoundByte:
 *  Place un octet dans le tampon arrire du streaming audio.
 */
static void PutSoundByte(unsigned long long int clock, unsigned char data)
{
    register int i;
    int index=(clock%TO7_CYCLES_PER_FRAME)*sound_freq/TO7_CPU_FREQ;

    /* Dans le cas o le nombre de cycles xcuts pendant une frame dpasse la valeur
       thorique, on bloque l'index  sa valeur maximale */
    if (index < last_index)
        index=sound_buffer_size;

    for (i=last_index; i<index; i++)
        sound_buffer[i]=last_data;

    last_index=index;
    last_data=data;
}



/* PlaySoundBuffer:
 *  Echange les tampons avant et arrire du streaming audio.
 */
void PlaySoundBuffer(void)
{
    register int i;
    char *buffer_ptr;

    /* on remplit la fin du buffer avec le dernier byte dpos */
    for (i=last_index; i<sound_buffer_size; i++)
        sound_buffer[i]=last_data;

    last_index=0;

    while ((buffer_ptr=get_audio_stream_buffer(stream)) == NULL)
        ;

    memcpy(buffer_ptr, sound_buffer, sound_buffer_size);
    
    free_audio_stream_buffer(stream);
}



/* voice_get_position_callback:
 *  Helper pour dtecter le bon fonctionnement du streaming audio.
 */
static void voice_get_position_callback(void)
{
    if (voice_get_position(stream->voice))
        thom.sound_enabled=TRUE;
}



/* InitSound:
 *  Initialise le module de streaming audio.
 */
void InitSound(int freq)
{
    sound_freq = freq;
    sound_buffer_size = sound_freq/TO7_FRAME_FREQ;
    sound_buffer = malloc(sizeof(unsigned char)*sound_buffer_size);

    to7_PutSoundByte=PutSoundByte;

    if (!thom.sound_enabled)
        return;

    thom.sound_enabled=FALSE;

    printf("Initialisation du son...");

    /* pas de compensation de volume */
    set_volume_per_voice(0);

    if (install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL) == 0)
    {
         /* test de fonctionnement du streaming */
         stream=play_audio_stream(sound_buffer_size, 8, FALSE, sound_freq, 128, 128);
         rest_callback(100, voice_get_position_callback);  /* 100 ms */
         voice_stop(stream->voice);
    }

    printf(thom.sound_enabled ? "ok\n" : "erreur\n");
}

