/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2002 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : keyboard.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou 1998
 *  Modifi par: Eric Botcazou 06/10/2001
 *
 *  Gestion du clavier (et des manettes) du TO7-70.
 */


#ifndef SCAN_DEPEND
   #include <stddef.h>
#endif

#include "intern/hardware.h"
#include "intern/keyboard.h"  /* MacOS */
#include "to7keys.h"
#include "to7.h"


#define NKEY_ROWS  8  /* nombre de lignes de touches du TO7-70 */

volatile int key_row[NKEY_ROWS]; /* ligne de touches du clavier */

static volatile int j0_dir[2], j1_dir[2]; /* buffer de direction des manettes */

static unsigned char key_code[KEY_MAX] = {
   0, 253,0x04, 253, 253, 253,0x08,0x08,0x08,0x10,0x04,0x04,0x04,0x01,0x01,0x10,
0x10, 253,0x20, 253,0x20,0x10,0x04, 253, 253,0x20, 253,0x40,0x80,0x80,0x80,0x80,
0x80,0x80,0x40,0x40,0x40,   0, 254, 254, 254, 254,   0, 254, 254, 254, 254,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0, 255,   0,0x40,0x40,   0,
0x80,   0,0x10,0x10,0x04,0x40,0x10,0x01,0x01,0x10,0x40,0x01,0x02,0x02,0x02,0x08,
   0,   0,0x02,0x02,0x02,0x02,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,0x01,0x01,0x20, 255,0x40,   0,   0,   0,   0,
   0,   0,   0
};
/* key_code[] convertit le scancode d'une touche PC (voir keycodes.h pour
    la liste des scancodes) en la valeur que produit le clavier TO7-70 pour 
    la mme touche; il y a trois valeurs particulires :
     - 0 dsigne une touche non mappe sur le TO7-70,
     - 253 dsigne une touche du bloc clavier gauche du PC,
     - 254 dsigne une touche du clavier numrique du PC,
     - 255 dsigne une touche spciale du PC mais n'a pas d'utilisation
       dans le traitement */

static unsigned char key_rcode[KEY_MAX] = {
   0,   5,   4,   6,   3,   3,   2,   1,   0,   1,   0,   1,   2,   1,   0,   2,
   3,   5,   2,   4,   1,   0,   5,   4,   5,   0,   6,   3,   5,   4,   3,   2,
   1,   0,   0,   1,   2,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   4,   5,   0,
   6,   0,  13,   6,   3,  12,   5,   3,   2,   4,   9,   4,   1,   0,   2,   6,
   0,   0,   5,   3,   6,   4,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   7,   7,   6,   0,   6,   0,   0,   0,   0,
   0,   0,   0
};

static unsigned char key_lpd_code[2][26]={ {
0x08,   0,0x04,0x08,0x20,  0,   0,   0,  0,   0,  0,  0,  0,  0,  0,  0,
0x20,   0,0x08,   0,   0,  0,0x20,0x01,  0,0x01}, {
0xb0,   0,0x50,0x70,0x60,  0,   0,   0,  0,   0,  0,  0,  0,  0,  0,  0,
0xa0,   0,  0 ,   0,   0,  0,0xe0,0xd0,  0,0x90}
};
/* key_lpd_code[] convertit le scancode d'une touche du bloc clavier gauche
    du PC en sa valeur clavier TO7-70 et en sa valeur manette TO7-70 */

static unsigned char key_pad_code[9]={0x9,0xd,0x5,0xb, 0,0x7,0xa,0xe,0x6};

static unsigned char key_shift[19]={ 36,61,63,41,46,47,
                                     64,38,35,34,39,40,45,32,33,94,
                                     47,46,62};


/* ResetJoypad:
 *  Remet au repos le port manettes.
 */
static void ResetJoypad(void)
{
    pia_ext.porta.idr = 0xFF;
    pia_ext.portb.idr = 0xCF;
    pia_ext.porta.cr  |= 0xC0;
    pia_ext.portb.cr  |= 0xC0;

    j0_dir[0]=j0_dir[1]=0xF;
    j1_dir[0]=j1_dir[1]=0xF0;
}



/* DoKeyShift:
 *  Transpose le code des touches shiftes.
 */
void DoKeyShift(int *br, int *cc)
{
    if (*cc&8)
    {
        if (*br==37)
        {
            *br=41;
            *cc&=0xF7;
        }
        else if ((42<=*br) && (*br<=60))
        {
            *br=key_shift[*br-42];
            *cc&=0xF7;
        }
    }
}



/* ResetKeyboard:
 *  Remet  zro le clavier et le port manette du TO7-70.
 */
void ResetKeyboard(int mask, int value)
{
    int i;

    for (i=0;i<NKEY_ROWS;i++)
        key_row[i]=0xFF;

    ResetJoypad();

    if (mask&TO7_SHIFT_FLAG)
        key_row[7] = (value&TO7_SHIFT_FLAG ? key_row[7]&0xFE : key_row[7]|0x01);

    if (mask&TO7_CTRL_FLAG)
        key_row[6] = (value&TO7_CTRL_FLAG ? key_row[6]&0xDF : key_row[6]|0x20);
 
    if (mask&TO7_CAPSLOCK_FLAG)
        mc6846.prc = (value&TO7_CAPSLOCK_FLAG ? mc6846.prc&0xF7 : mc6846.prc|0x08);
}



/* SetNoCapsLed:
 *  Allume/teint la led Minuscules.
 */
void SetNoCapsLed(int new_state)
{
    static int state=1;

    if (new_state^state)
    {
        state = new_state;
                
        if (to7_SetNoCapsLed)
            to7_SetNoCapsLed(state);
    }
}



/**********************************/
/* partie publique                */
/**********************************/


/* HandleKeyPress:
 *  Prend en compte la frappe ou le relchement d'une touche.
 *   key: scancode de la touche frappe/relache (voir to7keys.h pour la liste)
 *   release: flag d'enfoncement/relchement
 */
void to7_HandleKeyPress(int key, int release)
{
    unsigned char code;
    
    switch (key)
    {
        case KEY_LSHIFT:
        case KEY_RSHIFT:
            if (release)
                key_row[7] |= 0x01;
            else
                key_row[7] &= 0xFE;
            break;

        case KEY_RCONTROL:  /* le control droit mule le bouton joystick 0 */
            if (release)
            {
                pia_ext.portb.idr |= 0x40;
                pia_ext.porta.cr  |= 0x80;
            }
            else
            {
                pia_ext.portb.idr &= 0xBF;
                pia_ext.porta.cr  &= 0x7F;
            }
            break;

        case KEY_LCONTROL:  /* le controle gauche mule la touche CNT
                                et le bouton joystick 1 */
            if (release)
            {
                key_row[6] |= 0x20;
                pia_ext.portb.idr |= 0x80;
                pia_ext.portb.cr  |= 0x80;
            }
            else
            {
                key_row[6] &= 0xDF;
                pia_ext.portb.idr &= 0x7F;
                pia_ext.portb.cr  &= 0x7F;
            }
            break;

        default:
            code=key_code[key];

            if (code==253)  /* touche du bloc gauche */
            {
                if ((code=key_lpd_code[1][key-KEY_A]))
                {
                    if (release)
                    {
                        if (code == j1_dir[0])
                        {
                            j1_dir[0]=j1_dir[1];
                            j1_dir[1]=0xF0;
                        }
                        else if (code == j1_dir[1])
                            j1_dir[1]=0xF0;
                    }
                    else
                    {
                        if (code != j1_dir[0])
                        {
                            j1_dir[1]=j1_dir[0];
                            j1_dir[0]=code;
                        }
                    }

                    pia_ext.porta.idr = j1_dir[0] | j0_dir[0];
                }

                code=key_lpd_code[0][key-KEY_A];
            }
            else if (code==254)  /* touche du pav numrique */
            {
                code=key_pad_code[key-KEY_1_PAD];
       
                if (release)
                {
                    if (code == j0_dir[0])
                    {
                        j0_dir[0]=j0_dir[1];
                        j0_dir[1]=0xF;
                    }
                    else if (code == j0_dir[1])
                        j0_dir[1]=0xF;
                }
                else
                {
                    if (code != j0_dir[0])
                    {
                        j0_dir[1]=j0_dir[0];
                        j0_dir[0]=code;
                    }
                }

                pia_ext.porta.idr = j1_dir[0] | j0_dir[0];
                break; /* fin du traitement pour le pav numrique */
            }

	    if (code)  /* touche mappe */
            {
                if (key_rcode[key]&8)
                    key_row[7]^=0x01;

                if (release)
                    key_row[key_rcode[key]&7] |= code;
		else
                    key_row[key_rcode[key]&7] &= ~code;
            } 
    } /* end of switch */
}

END_OF_FUNCTION(to7_HandleKeyPress)



/* InitKeyboard:
 *  Effectue quelques modifications mineures de la routine de
 *  lecture clavier TO7-70.
 */
void InitKeyboard(void)
{
    /* on modifie l'emplacement de 4 caractres */
    mem.mon.bank[0][0x1D17]=37;  /* _ -> % */
    mem.mon.bank[0][0x1D19]=58;  /* . -> ; */
    mem.mon.bank[0][0x1D25]=60;  /* / -> : */
    mem.mon.bank[0][0x1D2D]=59;  /* @ -> < */

    /* appel de la routine de transposition des touches shiftes */
    mem.mon.bank[0][0x11F2]=0x02;

    LOCK_VARIABLE(key_row);
    LOCK_VARIABLE(key_code);
    LOCK_VARIABLE(key_rcode);
    LOCK_VARIABLE(key_lpd_code);
    LOCK_VARIABLE(key_pad_code);
    LOCK_VARIABLE(j0_dir);
    LOCK_VARIABLE(j1_dir);
    LOCK_FUNCTION(to7_HandleKeyPress);
}

